<?php
namespace Teknisa\PLA\Modules\Servico\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Ger04100Servico extends \Teknisa\GER\Controllers\TeknisaCrud {
	
	protected $servicoService;
	protected $crudService;
	protected $dataSourceManager;
	protected $util;
	protected $dtprocessa;
	protected $dataSourceName;
	protected $dataSourceNameToSave = false;
	protected $parameterBagColumns = array();
	protected $novoCodigo;
	protected $connection;
	protected $imgPathURL;

	public function __construct($servicoService, $crudService, $dataSourceManager, $util, $novoCodigo, $connection, $imgPathURL) {
		$this->servicoService = $servicoService;
		$this->crudService = $crudService;
		$this->dataSourceManager = $dataSourceManager;
		$this->util = $util;
		$this->novoCodigo = $novoCodigo;
		$this->connection = $connection;
		$this->imgPathURL = $imgPathURL;
	}
	
	public function salvarServico(DTO\Request $request, DTO\Response $response) {
		try {
			$this->dataSourceName = 'servico';
			$this->parameterBagColumns = array();
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$this->dtprocessa = $this->util->retornaDataProcessamento();
			foreach ($rows as $row) {
				$this->crudService->save($this->dataSourceName, array($row));
				if ($row['IDBLOQCONFIXPER'] == 'S') {
					$this->servicoService->deletaConsumo($row['CDFILIAL'], $row['CDSERVFILI'], $this->dtprocessa, $row['NRORG']);
				}
				if(isset($row['IDSERVTRAN']) && $row['IDSERVTRAN'] == 'N'){
					$this->servicoService->deletaEmbalagens($row);
				}
			}
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet("errorInfo", array("COMPLEMENTO" => $e->getMessage())));
			$response->setError(new Error("erroSalvarDados", 500));
		}
	}

	public function retornaTiposDePrato(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_tipos_de_prato';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTREFCUSPADR', 'NRORG');
		parent::find($request, $response);
	}

	public function retornaProximaDataCustoPadrao(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_proxima_data_custo_padrao';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG');
		parent::find($request, $response);
	}

	public function removeTipoDePrato(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_tipos_de_prato';
		parent::delete($request, $response);
	}

	public function removeTodosTiposDePrato(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->servicoService->deletaTodosTiposDePrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTREFCUSPADR'], $row['NRORG']);
	}

	public function removeTodosPontosEntregaCliente(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->servicoService->deletaTodosPontosEntregaCliente($row['CDFILIAL'], $row['CDSERVFILI'], $row['CDCLIENTE'], $row['NRORG']);
	}

	public function retornaPontosEntregaCliente(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_destfili';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG', 'CDCLIENTE');
		parent::find($request, $response);
	}

	public function retornaCustoServico(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_custo_servico';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI');
		parent::find($request, $response);
	}

	public function removeProdutosFaturamento(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'prodfaturam';
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
		$rows = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
		$dataSet = new DataSet($this->dataSourceName, (array)$rows);
		$this->crudService->delete($response, $dataSet);
	}

	public function findServico(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("ger04100_servico");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
	}

	private function setImgFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["FGFOTOSERVICO"] !== null) {
				$row["FGFOTOFILE"] = $row["FGFOTOSERVICO"];
				$row["FGFOTOSERVICO"] = $this->imgPathURL.$row["FGFOTOSERVICO"];
			}
		}
		return new DataSet("ger04100_servico", $rows);
	}

	public function saveImgServ(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		if (isset($row["IMAGE_UPLOAD"]) && isset($row["IMAGE_UPLOAD"]["0"]) 
		&& isset($row["IMAGE_UPLOAD"]["0"]["path"]) && $row["IMAGE_UPLOAD"]["0"]["path"] !== null) {
			$row["FGFOTOSERVICO"] = $row["IMAGE_UPLOAD"]["0"]["path"];
		} else if(isset($row["FGFOTOFILE"]) && $row["FGFOTOFILE"] !== null) {
			$row["FGFOTOSERVICO"] = $row["FGFOTOFILE"];
		}

		$newDataSet = new DataSet("servico", array($row));
		$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
		$rowsToSave = $this->crudService->getRowsToSave($newRequest, "servico");
		$this->crudService->save("servico", $rowsToSave);
	}
	
	public function exportConfigPadrao(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		try{
			
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			$servicoOrig = $this->getServico($cdFilialOrig, $cdServFiliOrig);
			
			if(isset($row['CDFILIAL_DESTINO']) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');	
			} else {
				$cdFilialDest = 'T';
			}
			
			if(isset($row['CDSERVFILI_DESTINO']) && 
					is_array($row->get('CDSERVFILI_DESTINO')) && 
					sizeof($row->get('CDSERVFILI_DESTINO'))>0){
					
				$cdServfiliDest = $row->get('CDSERVFILI_DESTINO');	
			} else {
				$cdServfiliDest = 'T';
			}
			
			if($row->get('__INCSERVUNIDEST')=='S') $this->exportIncServUniDest($row);
			if($row->get('__COPYPARAMSFORM')=='S') $this->exportParamsForm($row, $servicoOrig);
			
			$this->servicoService->exportConfigPadrao($cdFilialOrig, $cdServFiliOrig, $cdFilialDest, $cdServfiliDest);
			
		} catch(\Exception $e){
			
			$errorMessage = $e->getMessage();
			$message = '';
			
			if (strpos($errorMessage, 'ORA-02292') !== false) {
				if (strpos($errorMessage, 'FKPLA00440CONFCARD') !== false) {
					$message = "A exportação das configurações não foi realizada. Unidade destino possui registros filhos!";
				}
			} else {
				$message = $errorMessage;
			}
			
			$response->setError(new Error($message , 500));
		}
	}
	
	private function exportParamsForm($row, $servicoOrig){
		try{
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			if(isset($row['CDFILIAL_DESTINO']) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');	
			} else {
				$cdFilialDest = 'T';
			}
			
			
			if(isset($row['CDSERVFILI_DESTINO']) && 
					is_array($row->get('CDSERVFILI_DESTINO')) && 
					sizeof($row->get('CDSERVFILI_DESTINO'))>0){
				
				$cdServfiliDest = $row->get('CDSERVFILI_DESTINO');	
			} else {
				$cdServfiliDest = 'T';
			}
			
			$this->servicoService->exportParamsForm($cdServFiliOrig, $cdFilialOrig, $cdFilialDest, $cdServfiliDest, $servicoOrig);
		} catch(\Exception $e){
			throw $e;
		}
	}
	
	private function exportIncServUniDest($row){
		try{
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			if((null !== $row->get('CDFILIAL_DESTINO')) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');
				
			} else {
				$cdFilialDest = 'T';
			}
			
			$this->servicoService->exportIncServUniDest($cdServFiliOrig, $cdFilialOrig, $cdFilialDest);
		} catch(\Exception $e){
			throw $e;
		}
	}
	
	private function getServico($cdFilial, $cdServfili) {
        try{
			$conditions = array(
				array(
					"columnName" => "CDFILIAL",
					"operator"   => "=",
					"value"      => $cdFilial
				),
				array(
					"columnName" => "CDSERVFILI",
					"operator"   => "=",
					"value"      => $cdServfili
				)
			);
			$this->parameterBagColumns = array("CDFILIAL", "NRORG");
			
			$filterCriteria = new FilterCriteria('ger04100_servico', $conditions);
			$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
			
			$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
			
			$servico = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
			return sizeof($servico)>0 ? $servico[0] : array();
        } catch(\Exception $e){
        	throw $e;
        }
	}
	
	public function autoIncludeServico(DTO\Request $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			$originData = $row->get('ORIGIN_DATA');
			$targetData = $row->get('TARGET_DATA');
			
			$this->servicoService->autoIncludeServico($originData, $targetData);
        } catch(\Exception $e){
        	throw $e;
        }
	}
	
	public function getParamGeralIdutcardpadr(DTO\Request $request, DTO\Response $response){
		$Idutcardpadr = $this->servicoService->getParamGeralIdutcardpadr();
		$response->addDataSet(new DataSet('Idutcardpadr', $Idutcardpadr[0]));	
	}
	
	public function salvaEmbalagem(DTO\Request $request, DTO\Response $response){
		try{
            $this->connection->beginTransaction();

			$row = $request->getRow();
			$row['__is_new'] = true;
			if(empty($row['NRGRUPO'])) $row['NRGRUPO'] = $this->maxNrGrupo($row) + 1;	

        	foreach ($row['NMTIPOPRAT'] as $nrseqcnfcard) {
        		$row['NRCNFDISTEMBSERV'] = $this->novoCodigo->getCodeByTableName('CNFDISTEMBSERV',$row['NRORG']);
	        	$row['NRSEQCNFCARD'] = $nrseqcnfcard;
	        	$newRequest = new DTO\Request\Row($row,"","","");
	            $dataSourceName = 'cnfdistembserv';
	            $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
	            $this->crudService->save($dataSourceName,$rowToSave);
        	}
           
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack();
           $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	public function excluiEmbalagens(DTO\Request $request, DTO\Response $response){
		try{
			$rows =  $request->getRow();
            $this->connection->beginTransaction();
			foreach($rows as $row){
				$data = $this->filtraPratosEmb($row);
				$this->excluiCnfDistEmbServ($data);
			}
			$this->connection->commit();
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}

	private function filtraPratosEmb($row){
		if(isset($row['NRCNFDISTEMBSERV'])) $nrcnfdistembserv = $row['NRCNFDISTEMBSERV'];
		else $nrcnfdistembserv = null;
		
		$params = array(
            'CDFILIAL'			=>  $row['CDFILIAL'],
        	'CDSERVFILI'		=>  $row['CDSERVFILI'],
        	'NRGRUPO'			=>  $row['NRGRUPO'],
        	'CDCNFDISTEMBSERV'  =>  $row['CDCNFDISTEMBSERV'],
        	'CDDESTFILI'		=>  $row['CDDESTFILI'],
        	'NRCNFDISTEMBSERV'  =>	$nrcnfdistembserv
        );

        $sql = "SELECT CC.CDTIPOPRAT, CNF.* FROM CNFDISTEMBSERV CNF
				INNER JOIN CONFCARD CC
				ON CC.CDFILIAL = CNF.CDFILIAL
				AND CC.CDSERVFILI = CNF.CDSERVFILI
				AND CC.NRSEQCNFCARD = CNF.NRSEQCNFCARD
				WHERE CNF.CDFILIAL =:CDFILIAL AND CNF.CDSERVFILI =:CDSERVFILI AND
					  ((CNF.CDDESTFILI IS NULL AND :CDDESTFILI IS NULL) OR CNF.CDDESTFILI =:CDDESTFILI) AND
    				  (:CDCNFDISTEMBSERV IS NULL OR CNF.CDCNFDISTEMBSERV =:CDCNFDISTEMBSERV) AND 
                      (:NRGRUPO IS NULL OR CNF.NRGRUPO =:NRGRUPO) AND
    				  (:NRCNFDISTEMBSERV IS NULL OR INSTR(:NRCNFDISTEMBSERV ,CNF.NRCNFDISTEMBSERV)>0)
    			ORDER BY CNF.CDCONTAINER";
	        
        $data = $this->connection->fetchAll($sql, $params);
        return $data;
	}
	
	private function excluiCnfDistEmbServ($data){
		$dataSourceName = "cnfdistembserv";
	    foreach ($data as $row) {
			$newRequest = new DTO\Request\Row($row,"","","");
	     	$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	     	$dataSet = new DataSet($dataSourceName, $rowToDelete);
	        $this->dataSourceManager->delete($dataSet);
		}
	}
	
	private function maxNrGrupo($row){
		$params = array(
            'CDFILIAL'	  =>  $row['CDFILIAL'],
        	'CDSERVFILI'  =>  $row['CDSERVFILI']
        );
        $sql = "SELECT MAX(NRGRUPO) AS NRGRUPO FROM CNFDISTEMBSERV WHERE CDFILIAL =:CDFILIAL AND CDSERVFILI =:CDSERVFILI";
        $data = $this->connection->fetchAll($sql, $params);
        return $data[0]['NRGRUPO'];
	}
	
	public function excluiEmbalagensConfPadrao(DTO\Request $request, DTO\Response $response){
		try{
			$rows =  $request->getRow();
            $this->connection->beginTransaction();
			
			foreach($rows as $row){
	            $query="DELETE FROM CNFDISTEMBSERV WHERE CDFILIAL =:CDFILIAL AND CDSERVFILI =:CDSERVFILI AND NRSEQCNFCARD =:NRSEQCNFCARD";
	            $this->connection->executeQuery($query,
	            array(
	            	"CDFILIAL" => $row['CDFILIAL'],
	            	"CDSERVFILI" => $row['CDSERVFILI'],
	            	"NRSEQCNFCARD" => $row['NRSEQCNFCARD']
	            	)
	            );
			}
			$this->connection->commit();
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}

	public function getPratoOrigem($row){
		
	    $params = array(
            'FILIAL_ORIGEM'	 =>  $row['CDFILIALORI'],
        	'SERVICO_ORIGEM' =>  $row['CDSERVFILIORI'],
        	'CDDESTFILI_ORIGEM' => $row['CDDESTFILI']
        );
        $sql = "SELECT 
					CC.CDTIPOPRAT, CNF.CDFILIAL, CNF.CDDESTFILI, CNF.CDCONTAINER, 
					CO.NMCONTAINER, TP.NMTIPOPRAT, CNF.NRSEQCNFCARD, CNF.NRCNFDISTEMBSERV, 
					CNF.CDCNFDISTEMBSERV
				FROM 
					CNFDISTEMBSERV CNF 
				LEFT JOIN CONFCARD CC 
					ON 
						CC.CDFILIAL = CNF.CDFILIAL 
						AND CC.CDSERVFILI = CNF.CDSERVFILI 
						AND CC.NRSEQCNFCARD = CNF.NRSEQCNFCARD 
				LEFT JOIN TIPOPRAT TP 
					ON 
						CC.CDTIPOPRAT = TP.CDTIPOPRAT 
				LEFT JOIN CONTAINER CO 
					ON 
						CO.CDCONTAINER = CNF.CDCONTAINER 
				WHERE 
					CNF.CDFILIAL = :FILIAL_ORIGEM 
					AND CNF.CDSERVFILI = :SERVICO_ORIGEM 
					AND ( 
						( CNF.CDDESTFILI IS NULL AND :CDDESTFILI_ORIGEM IS NULL ) 
						OR (CNF.CDDESTFILI IS NOT NULL AND :CDDESTFILI_ORIGEM IS NOT NULL AND CNF.CDDESTFILI = :CDDESTFILI_ORIGEM) )";
	        
        $data = $this->connection->fetchAll($sql, $params);
        return $data;
		
	}
	
	public function validaPratoDestino($row, $pratoOrigem){
		 		 
		 $pratoValida = null;
		 $pratoInvalido = null;
		 $servicoDest=[];
		 
		 if(is_array($row['NMSERVFILIDEST'])){
		 	$servicoDest = $row['NMSERVFILIDEST'];
		 }else{
		 	$servicoDest[] = substr($row['NMSERVFILIDEST'],0,5);
		 }
		 $tipoEmbalagem = [
			 "DIST" => "DISTRIBUIÇÃO AUTOMÁTICA",
			 "DISTR" => "DISTRIBUIÇÃO AUTOMÁTICA RECURSIVA",
		 ];
		 foreach ($servicoDest as $servicoDestino) {
		 
			 foreach ($pratoOrigem as $prat) {
  			 
			    $params = array(
		            'FILIAL_DESTINO'	=>  $row['CDFILIALDEST'],
		        	'SERVICO_DESTINO'	=>  $servicoDestino,
		        	'PRATO_ORIGEM'		=>  $prat['CDTIPOPRAT']
		        
		        );
	
		        $sql = "SELECT CC.CDTIPOPRAT, CC.NRSEQCNFCARD FROM CONFCARD CC
						WHERE 
							CC.CDFILIAL = :FILIAL_DESTINO
							AND CC.CDSERVFILI = :SERVICO_DESTINO
							AND CC.CDTIPOPRAT = :PRATO_ORIGEM";
			        
		        $data = $this->connection->fetchAll($sql, $params);
		        
		        if(!empty($data)){
		        	$pratoValida[]	= $prat['NRCNFDISTEMBSERV'];

		        }else{
		        	$pratoInvalido[] = [
						"CDFILIAL" => $row['CDFILIALDEST'],
						"CDSERVFILI" => $servicoDestino,
						"CDCONTAINER" => $prat['CDCONTAINER'] ?? $prat['CDCNFDISTEMBSERV'],
						"NMCONTAINER" => $prat['NMCONTAINER'] ?? $tipoEmbalagem[$prat['CDCNFDISTEMBSERV']],
						"NMTIPOPRAT" => $prat['NMTIPOPRAT'],
					]; 
		        }
	
			 }
		 }   

		 if(!empty($pratoValida)){
			$result['PRATO_VALIDO'] = implode("_", $pratoValida);
		 }

		 if(!empty($pratoInvalido)){
			$result['PRATO_INVALIDO'] = $pratoInvalido;
		}

		return $result;
			
	}	
		
	
	public	function getConfigPadraoServ($CDFILIAL, $CDSERVFILI) {
	    $params = array(
			'CDFILIAL'	 =>  $CDFILIAL,
			'CDSERVFILI' =>  $CDSERVFILI
		);
		$sql = "SELECT CDSERVFILI, CDTIPOPRAT, NRSEQCNFCARD FROM CONFCARD 
				WHERE CDFILIAL = :CDFILIAL 
						AND CDSERVFILI = :CDSERVFILI";
							
		$confserv = $this->connection->fetchAll($sql, $params);
	    return $confserv;
	}

	public	function getMapConfig($config){
	    $mapConfig = [];
	    foreach($config as $conf) {
	        if(!isset($mapConfig[$conf['CDTIPOPRAT']])) $mapConfig[$conf['CDTIPOPRAT']] = [];
	        $mapConfig[$conf['CDTIPOPRAT']][] = $conf['NRSEQCNFCARD'];
	    }
	    return $mapConfig;
	}
	
	
	public	function validaPratoInvalidoPontoEntrega($pratosInvalidos, $servicoSemPontoEntrega){

	    $pratosInv = [];
	    $contPratosInvalidos = 0;
	    
	    foreach($pratosInvalidos as $p){
	    	
	    	foreach($servicoSemPontoEntrega as $s){
	    	
	    		$verificaPratoInv = array_search($p['CDSERVFILI'], $s);
	    		
	    		if(!empty($verificaPratoInv)){
	    			$contPratosInvalidos = 1;
	    		}
   		
	    	}
   	
	    	if($contPratosInvalidos == 0){
	    		$pratosInv[] = $p;
	    	}
	    	
	    	$contPratosInvalidos = 0;
	    	
	    }
	    
	    return $pratosInv;
	}
	

	public function salvaImportExport(DTO\Request $request, DTO\Response $response){
	
		try{
			$row =  $request->getRow();
            $this->connection->beginTransaction();
        	$row['CDFILIAL'] = $row['CDFILIALORI'];
    		$row['CDSERVFILI'] = $row['CDSERVFILIORI'];
    		$row['CDDESTFILI'] = empty($row['CDDESTFILIORI']) ? null : $row['CDDESTFILIORI'] ;
			$row['NRGRUPO'] = null;
			$row['CDCNFDISTEMBSERV'] = null;

			$pratoOrigem = $this->getPratoOrigem($row);
			
			if(!empty($pratoOrigem)){
				$validacaoPrato = $this->validaPratoDestino($row, $pratoOrigem);
			}
			
			if(!empty($validacaoPrato)){
				if(!empty($validacaoPrato['PRATO_VALIDO'])){
					$row['NRCNFDISTEMBSERV'] = 	$validacaoPrato['PRATO_VALIDO'];	
				}
			}
			
			$pratoValido = $validacaoPrato['PRATO_VALIDO'] ?? [];
			$pratoInvalido = $validacaoPrato['PRATO_INVALIDO'] ?? [];

			if($row['OPERATION'] == 'IMPORT'){
				$defaultConfigDest = $this->getConfigPadraoServ($row['CDFILIALDEST'], $row['CDSERVFILIDEST']);
				if(empty($defaultConfigDest)) {
					throw new \Exception('DESTINO_SEM_CONFIGURACAO_PADRAO');
				}
				
				if(empty($pratoInvalido) && empty($pratoValido)){
					throw new \Exception('origemSemRegistros');
				}
				if(!empty($pratoInvalido) && empty($pratoValido)){
					throw new \Exception('CONFIGURACAO_PADRAO_DIFERENTE');
				}
				
			} else if($row['OPERATION'] == 'EXPORT' && is_array($row['NMSERVFILIDEST']) && (!is_array($row['CDDESTFILIDEST']) || empty($row['CDDESTFILIDEST']))){
				$servicos = '_'.implode('_', $row['NMSERVFILIDEST']).'_';
				$conditions = [
					[
						"columnName"=> "CDFILIAL",
						"operator"	=> "=",
						"value" 	=> $row['CDFILIALDEST']
					],
					[
						"columnName"=> "CDSERVFILI",
						"operator"	=> "=",
						"value" 	=> $servicos
					]
				];
				$pontoEntrega = $this->crudService->getFilteredDataSet("ger04100_ponto_entrega_servfili", $conditions)->getRows();
				$row['CDDESTFILIDEST'] = array_column($pontoEntrega, 'CDSERVDESTCLI');
				
				$validaPontoEntrega = [];
				$possuiPontoEntrega = [];
				$naoPossuiPontoEntrega = [];
				
				//Verifica se destino tem ponto de entrega quando $row['COPIARDISTNIVSERV'] == 'N'
				foreach($row['NMSERVFILIDEST'] as $servDest){
					if(!empty($pontoEntrega)){
						$conditions = [
							[
								"columnName"=> "CDFILIAL",
								"operator"	=> "=",
								"value" 	=> $row['CDFILIALDEST']
							],
							[
								"columnName"=> "CDSERVFILI",
								"operator"	=> "=",
								"value" 	=> $servDest
							]
						];
						$existePontoEntrega = $this->crudService->getFilteredDataSet("ger04100_cliente", $conditions)->getRows();
						if(!empty($existePontoEntrega)){
							$possuiPontoEntrega[] = $servDest;	
						}else{
							$naoPossuiPontoEntrega[] = [
									"CDSERVFILI" => $servDest,
									"CDFILIALDEST" =>$row['CDFILIALDEST']
							];
						}
					}else{
						$naoPossuiPontoEntrega[] = [
							"CDSERVFILI" => $servDest,
							"CDFILIALDEST" =>$row['CDFILIALDEST']
						];
					}
					
				}
					$validaPontoEntrega['EXISTE_PONTO_ENTREGA'] = $possuiPontoEntrega ?? [];
					$validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA'] = $naoPossuiPontoEntrega ?? [];
					
				
			}
		
			$servicoDest = [];
			$resultEmbOri = [];
			if(is_array($row['NMSERVFILIDEST'])){
				$servicoDest = $row['NMSERVFILIDEST'];
			}else{
				$servicoDest[] = substr($row['NMSERVFILIDEST'],0,5);
			}
			foreach($servicoDest as $servDestino) {
				$config = $this->getConfigPadraoServ($row['CDFILIALDEST'], $servDestino);
			    $mapConfigDestino = $this->getMapConfig($config);
				
			    
			    $dataOri = $this->filtraPratosEmb($row);
			    if(empty($config)) continue; // serviço não possui configuração
				
			    if(!empty($validacaoPrato)){
					$row['NRCNFDISTEMBSERV'] = null;
			    }
			    foreach($dataOri as $emb) {
					$opcoesDestino = $mapConfigDestino[$emb['CDTIPOPRAT']] ?? [];
					
			        if(!empty($opcoesDestino)) { // ainda existem colunas no servico destino para atribuir a embalagem
			            $emb['NRSEQCNFCARD'] = array_shift($mapConfigDestino[$emb['CDTIPOPRAT']]);
			            $emb['SERV_DESTINO'] = $servDestino;
			            unset($emb['CDTIPOPRAT']); // CDTIPOPRAT não é uma coluna da tabela onde os dados seram inseridos (CNFDISTEMBSERV)
			            $resultEmbOri[] = $emb;
			        }
			    }
			}

			if(!empty($validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA']) && empty($validaPontoEntrega['EXISTE_PONTO_ENTREGA']) && $row['COPIARDISTNIVSERV'] == 'N'){ //Tela esta configurada para exportar aos pontos de entrega mas destino não possui nenhum ponto de entrega.
				throw new \Exception('NENHUM_PONTO_ENTREGA');
			}
			
		
			if(!empty($dataOri)){
				if(is_array($row['CDDESTFILIDEST'])){
					$rowsToSave = [];
					foreach ($row['CDDESTFILIDEST'] as $destino) {
						$row['CDSERVFILIDEST']  = substr($destino,0,5);
						$row['CDDESTFILIDEST']  = empty(substr($destino,5,3)) ? null : substr($destino,5,3);
						$row['CDCLIENTEDEST']	= empty(substr($destino,8,15)) ? null : substr($destino,8,15);
						$row['CDFILIAL'] = $row['CDFILIALDEST'];
						$row['CDSERVFILI'] = $row['CDSERVFILIDEST'];
						$row['CDDESTFILI'] = empty($row['CDDESTFILIDEST']) || !isset($row['CDDESTFILIDEST']) ? null : $row['CDDESTFILIDEST'] ;
						$row['CDCLIENTE'] = empty($row['CDCLIENTEDEST']) || !isset($row['CDCLIENTEDEST']) ? null : $row['CDCLIENTEDEST'];
						$row['NRGRUPO'] = null;
						$row['CDCNFDISTEMBSERV'] = null;

				
						$dataDest = $this->filtraPratosEmb($row);
						if(!empty($dataDest))$this->excluiCnfDistEmbServ($dataDest);
						
						foreach($resultEmbOri as $embOrigem){
							if($embOrigem['SERV_DESTINO'] == $row['CDSERVFILI']){
								$dataEmbOrigem[] = $embOrigem;
							}
						}
						if(!empty($dataEmbOrigem)){
							$rowsToSave = array_merge($rowsToSave, $this->salvaDestino($dataEmbOrigem, $row, true));
						}
						$dataEmbOrigem = [];
					}
					if(!empty($rowsToSave)){
						$dataSourceName = 'cnfdistembserv';
						$this->crudService->save($dataSourceName, $rowsToSave);
					}
				}
				if($row['COPIARDISTNIVSERV'] == 'S' ){
					$rowsToSave = [];
					foreach($row['NMSERVFILIDEST'] as $servico){
						$row['CDFILIAL'] = $row['CDFILIALDEST'];
		        		$row['CDSERVFILI'] = $servico;
						$row['CDDESTFILI'] = null;
		    			$row['CDCLIENTE'] = null;
		    			$dataDest = $this->filtraPratosEmb($row);
		    			if(!empty($dataDest))$this->excluiCnfDistEmbServ($dataDest);
		    			
		    			foreach($resultEmbOri as $embOrigem){
							if($embOrigem['SERV_DESTINO'] == $row['CDSERVFILI']){
								$dataEmbOrigem[] = $embOrigem;
							}
						}
						
						if(!empty($dataEmbOrigem)){
							$rowsToSave = array_merge($rowsToSave, $this->salvaDestino($dataEmbOrigem, $row, true));
						}
						$dataEmbOrigem = [];
					}
					if(!empty($rowsToSave)){
						$dataSourceName = 'cnfdistembserv';
						$this->crudService->save($dataSourceName, $rowsToSave);
					}
				}
				
			}else if(empty($dataOri)){
				$response->addDataSet(new DataSet ('naoPossuiEmbalagem', array($row)));
			}

			$dadosSalvosEmb = isset($rowsToSave) ? $rowsToSave : [];

            if(!empty($validacaoPrato['PRATO_INVALIDO'])){
                    if($row['COPIARDISTNIVSERV'] == 'N' && !empty($validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA'])){
                        $validaPratosInv = $this->validaPratoInvalidoPontoEntrega($validacaoPrato['PRATO_INVALIDO'],$validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA']);
                        if(!empty($validaPratosInv)){
                            $response->addDataSet(new DataSet ('PRATO_INVALIDO',$validaPratosInv));
                            $response->addDataSet(new DataSet ('DADOS_SALVOS',$dadosSalvosEmb));
                        }
                    }else{
                            if(!empty($validacaoPrato['PRATO_INVALIDO'])){
                                $response->addDataSet(new DataSet ('PRATO_INVALIDO',$validacaoPrato['PRATO_INVALIDO']));
                                $response->addDataSet(new DataSet ('DADOS_SALVOS',$dadosSalvosEmb));
                            }
                    }
            }

            if(!empty($validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA'])){
                $response->addDataSet(new DataSet ('NAO_EXISTE_PONTO_ENTREGA', $validaPontoEntrega['NAO_EXISTE_PONTO_ENTREGA']));
                $response->addDataSet(new DataSet ('DADOS_SALVOS',$dadosSalvosEmb));
            }
		$this->connection->commit();
			
			
				
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}
	
	private function salvaDestino($dataOri, $row, $onlyPrepareRow = false){
		$novoNrgrupo = $this->maxNrGrupo(array('CDFILIAL' => $row['CDFILIAL'], 'CDSERVFILI' => $row['CDSERVFILI'])) + 1;
		$nrgrupoOri = $dataOri[0]['NRGRUPO'];
		$rowsToSave = [];
		foreach($dataOri as $rowOri){
			if($rowOri['NRGRUPO'] != $nrgrupoOri){
				$nrgrupoOri = $rowOri['NRGRUPO'];
				$novoNrgrupo++;
			}
			
			$rowOri['__is_new'] = true;
			$rowOri['CDFILIAL'] = $row['CDFILIAL'];
			$rowOri['CDSERVFILI'] = $row['CDSERVFILI'];
			$rowOri['CDDESTFILI'] = $row['CDDESTFILI'];
			$rowOri['CDCLIENTE'] = $row['CDCLIENTE'];
			$rowOri['NRGRUPO'] = $novoNrgrupo;
			$rowOri['NRCNFDISTEMBSERV'] = $this->novoCodigo->getCodeByTableName('CNFDISTEMBSERV', $rowOri['NRORG']);
			$newRequest = new DTO\Request\Row($rowOri,"","","");
            $dataSourceName = 'cnfdistembserv';
			
            $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
			$rowsToSave[] = $rowToSave[0];
			if(!$onlyPrepareRow){
            	$this->crudService->save($dataSourceName,$rowToSave);
			}
		}
		return $rowsToSave;
	}
	
	public function verificaCodigoExistente($request, $response){
		$row = $request->getRow();
		 $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator" => "=",
                "value" => $row['CDSERVPADR']
            )
        );
        
        $dataset = $this->crudService->getFilteredDataSet('servico', $conditions);
        $servico = $dataset->getRows();
        
        $response->addDataSet(new DataSet ('existe', array(!empty($servico))));
	}
	
	private function isValidServFat($row){
		$query = "
			SELECT MAX(CDFILIAL) AS CDFILIAL, MAX(DTDATA) AS DTDATA 
				FROM(SELECT MAX(CDFILIAL) AS CDFILIAL, 
					MAX(DTEFETCLIE) AS DTDATA 
				FROM EFETCLIE 
				WHERE CDFILIAL = :CDFILIAL 
					AND CDSERVFILI = :CDSERVFILI 
					AND CDSERVPLAN = :CDSERVPLAN 
				UNION ALL SELECT MAX(CDFILIAL) AS CDFILIAL, 
					MAX(DTFATURAM) AS DTDATA 
				FROM EFECLIFAT WHERE CDFILIAL = :CDFILIAL 
					AND CDSERVFILI = :CDSERVFILI 
					AND CDSERVPLAN = :CDSERVPLAN
			)
		";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDSERVFILI' => $row['CDSERVICFATURAM'],
			'CDSERVPLAN' => $row['CDSERVFILI']
		);
		$std = $this->connection->prepare($query);
		$std->execute($params);
		$rowResult = $std->fetch();
		return $rowResult['CDFILIAL'] === NULL;
	}
	
	public function deleteServFat(DTO\Request $request, DTO\Response $response){
		$dataSourceName = 'ger04100_servico_faturamento';
		$validServices = array();
		$request = $this->crudService->buildDataSetIfRow($request, $dataSourceName);
		$dataSetRequest = $request->getDataSet()->getRows();
		$invalidServices = array();
		
		foreach($dataSetRequest as $currentService){
			if($this->isValidServFat($currentService)){ 
				array_push($validServices, $currentService);
			} else {
				$invalids = array (
					"CDSERVFATURAMENTO" => $currentService['CDSERVICFATURAM'],
					"NOMESERVICO" => $currentService['NMSERVICO']
				);
				array_push($invalidServices, $invalids);
			}
				
		}
			if(!empty($invalidServices)){
				$response->addDataSet(new DataSet("invalidService", $invalidServices));
			}
			if(!empty($validServices)){
				$request = new DTO\Request\DataSet(new DataSet("Deleted", $validServices), $request->getMethod(), $request->getRoutepath(), $request->getUserId());
				$rows = $this->crudService->getRowsToDelete($request, $dataSourceName);
				$dataSet = new DataSet($dataSourceName, (array)$rows);
				$this->crudService->delete($response, $dataSet);
			}
	}	
	
	public function retornaDtProcessaPorFilial(DTO\Request\Row $request, DTO\Response $response){
		$query = "SELECT TO_CHAR(PF.DTPROCESSA, 'DD/MM/YYYY') AS DTPROCESSA, QTDIABLOQCAR FROM PARAMFILIAL PF JOIN PARAMGERAL PG ON PG.NRORG = PF.NRORG WHERE PF.CDFILIAL = :CDFILIAL";
		$row = $request->getRow();
		$params = array(
			'CDFILIAL' => $row['CDFILIAL']
		);
		$std = $this->connection->prepare($query);
		$std->execute($params);
		$response->addDataSet(new DataSet('DATA', $std->fetch()));
	}
	
	private function getAllDietConcat($row){
		$query = "SELECT
					    LISTAGG(DIETSERV.CDDIETPAD, '_') WITHIN GROUP(
					        ORDER BY
					            DIETSERV.CDDIETPAD
					    ) AS ALL_CDDIETPAD
					FROM
					    DIETSERV,
					    DIETPADR
					WHERE
					    ( DIETSERV.CDFILIAL = :CDFILIAL )
					    AND ( DIETSERV.CDSERVFILI = :CDSERVFILI )
					    AND ( DIETSERV.CDDIETPAD = DIETPADR.CDDIETPAD )";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDSERVFILI' => $row['CDSERVFILI']
		);
		$std = $this->connection->prepare($query);
		$std->execute($params);
		return $std->fetch()['ALL_CDDIETPAD'];
	}
	
	public function exportaComensalCardapio(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		if(!isset($row['CDDIETPAD']) || empty($row['CDDIETPAD'])){
			$row['LDIETA'] = $this->getAllDietConcat($row);
		} else {
			$row['LDIETA'] = implode('_', $row['CDDIETPAD']);
		}
		if($row['LDIETA']){
			$row['LDIETA'] = '_'.$row['LDIETA'].'_';
		}
		$this->servicoService->exportaComensalCardapio($row);
	}
	
	public function saveConfCardGrupoPrato($request, $response){
		$dataSourceName = 'confcardgrupoprato';

		$data = $request->getRow()['GRUPO_PRATO_EXPORT'];
		$dataBeforeChange = $request->getRow()['dataBeforeChange'];
		$changedRows = $request->getRow()['changedRows'];
		$serviceData = $request->getRow()['servData'];
		$unmarkedRows = $request->getRow()['unmarkedRows'];
		
		for ($i=0; $i < sizeof($data); $i++) {		
			$newGroupAssociation = array(
				"NRCONFCARDGRUPOPRATO" => $this->novoCodigo->getCodeByTableName('CONFCARDGRUPOPRATO', $serviceData['NRORG']),
				"CDFILIAL" =>  $serviceData['CDFILIAL'],
				"CDSERVFILI" =>  $serviceData['CDSERVFILI'],
				"NRSEQCNFCARD" =>  $serviceData['NRSEQCNFCARD'],
				"NRGRUPOPRATOSERV" =>  $data[$i],
				"NRORG" => $serviceData['NRORG'],
				"__is_new" => true
			);
			
			$newGroupAssociation = new DTO\Request\Row($newGroupAssociation,"","","");	
			$rowToSave = $this->crudService->getRowsToSave($newGroupAssociation, $dataSourceName);
			$this->crudService->save($dataSourceName,$rowToSave);
		}
		for ($i=0; $i < sizeof($unmarkedRows); $i++) {
			$newRequest = new DTO\Request\Row(array(
				"NRCONFCARDGRUPOPRATO" => $unmarkedRows[$i]
			),"","","");
			$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
			$dataSet = new DataSet($dataSourceName, $rowToDelete);
			$this->dataSourceManager->delete($dataSet);
		}
	}
	
}